@@ -279,7 +279,7 @@ class RhythmAddOnClass:
         """
         # FIXME can we use lessonfile.Rhythm insted of this?
         score = mpd.parser.parse_to_score_object(rhythm)
-        track = score.get_midi_events_as_percussion()[0]
+        track = mpd.score_to_percussion_tracks(score)[0]
         track.prepend_bpm(self.get_int("bpm"))
         track.prepend_volume(cfg.get_int('config/preferred_instrument_volume'))
         track.replace_note(mpd.notename_to_int("c"),
