@@ -42,6 +42,7 @@ from solfege import utils
 from solfege import uuid
 from solfege import xrandom
 from solfege.dataparser import istr
+from solfege.mpd import elems
 from solfege.mpd import mpdutils
 from solfege.mpd.duration import Duration
 from solfege.mpd.rat import Rat
@@ -164,6 +165,7 @@ keywords = (
     'idproperty',
     'rhythmdictation',
     'rhythmdictation2',
+    'tuner',
     #
     'harmonic',
     'melodic',
@@ -376,111 +378,6 @@ class _Header(dict):
             return self[name]
         return istr("")
 
-class Bar(list):
-    class Item(object):
-        """
-        An object representing either a note or a rest.
-        """
-        def __init__(self, isnote, duration, tie):
-            assert isinstance(duration, Duration)
-            self.m_isnote = isnote
-            self.m_duration = duration
-            self.m_tie = tie
-    class BadDuration(Exception):
-        pass
-    class FullException(Exception):
-        pass
-    stepwidth = 30
-    def __init__(self, keysig):
-        assert isinstance(keysig, Rat)
-        self.m_keysig = keysig
-    def _do_add(self, isnote, n):
-        if not isinstance(n, Duration):
-            if not isinstance(n, unicode):
-                raise self.BadDuration(n)
-            n = Duration.new_from_string(n)
-        if self.duration() + n.get_rat_value() > self.m_keysig:
-            raise self.FullException()
-        self.append(self.Item(isnote, n, False))
-    def isfull(self):
-        return self.duration() == self.m_keysig
-    def can_append(self, item):
-        """
-        Return True if the bar is not full and there is space to add item.
-        """
-        return (self.duration()
-                + item.m_duration.get_rat_value() <= self.m_keysig)
-    def add_note(self, n):
-        self._do_add(True, n)
-    def add_rest(self, n):
-        self._do_add(False, n)
-    def duration(self):
-        r = Rat(0)
-        for item in self:
-            r += item.m_duration.get_rat_value()
-        return r
-    def do_layout(self, show_timesig):
-        self.m_show_timesig = show_timesig
-        if show_timesig:
-            xpos = self.stepwidth
-        else:
-            xpos = self.stepwidth / 2
-        for i in self:
-            i.m_xpos = xpos
-            xpos += 30
-        #If the bar is full, then the width of the bar is the m_xpos
-        #of the last note + Bar.stepwidth
-        if self.isfull():
-            self.m_width = self[-1].m_xpos + Bar.stepwidth
-        self.m_width = xpos + int((self.m_keysig - self.duration()) * 4 * Bar.stepwidth)
-    def _get_timelist(self):
-        """
-        A timelist is a list of list:
-        [isnote, duration] of the types [bool, Rat]
-        """
-        v = []
-        tie = False
-        for item in self:
-            if tie:
-                v[-1][1] += item.m_duration.get_rat_value()
-            else:
-                v.append([item.m_isnote, item.m_duration.get_rat_value()])
-            tie = item.m_tie
-        return v
-
-
-class RhythmStaff(list):
-    """
-    A staff is a list of Bars
-    A object that know how long rhythm to enter, and will
-    know of all time signature changes and bar lines.
-
-    A question in the lesson file could be written like this:
-    \time 3/4 c4 c8 c8 c4 | \time 3/8 c8 c8 c16 c16
-    or:
-    4 8 8 4 | 8 8 16 16
-
-    We need code that can transform the former to the last format so
-    that we can compare the real music to the rhythm entered by
-    the user.
-    """
-    def do_layout(self):
-        timesig = None
-        xpos = 0
-        for bar in self:
-            bar.m_xpos = xpos
-            bar.do_layout(timesig != bar.m_keysig)
-            timesig = bar.m_keysig
-            xpos += bar.m_width
-    def get_timelist(self):
-        v = []
-        for bar in self:
-            v.extend(bar._get_timelist())
-        return v
-    def append(self, bar):
-        if not isinstance(bar, Bar):
-            raise TypeError("RhythmStaff.append: only Bar allowed")
-        list.append(self, bar)
 
 class MusicBaseClass(object):
     """
@@ -577,33 +474,37 @@ class MpdParsable(MusicBaseClass):
         Create and return a RhythmStaff showing the rhythm of the music
         object.
         """
-        staff = RhythmStaff()
+        score = elems.Score()
+        staff = score.add_staff(staff_class=elems.RhythmStaff)
         timesig = None
-        bar_content = Rat(0, 1)
         # the current note/rest-duration
         duration = Duration(4, 0)
+        tie_is_in_the_air = False
         for toc_type, toc in mpd.parser.Lexer(
                 self.get_mpd_music_string(lessonfile_ref)):
             if toc_type in (mpd.parser.Lexer.NOTE, mpd.parser.Lexer.REST):
                 if timesig is None:
                     # very first note and no timesig parsed
-                    staff.append(Bar(Rat(4, 4)))
-                    timesig = Rat(4, 4)
-                if staff[-1].isfull():
-                    staff.append(Bar(timesig))
+                    score.add_bar(elems.TimeSignature(4, 4))
+                    timesig = elems.TimeSignature(4, 4)
+                if score.voice11.is_bar_full():
+                    score.add_bar(timesig)
                 if toc.m_duration:
                     duration = toc.m_duration
-                staff[-1].append(Bar.Item(toc_type == mpd.parser.Lexer.NOTE,
-                                duration, False))
+                score.voice11.append(elems.Note(toc.m_pitch, duration))
+                if tie_is_in_the_air:
+                    score.voice11[-2]['elem'][0].m_tieinfo = 'start'
+                    score.voice11[-1]['elem'][0].m_tieinfo = 'end'
+                    tie_is_in_the_air = False
             elif toc_type == mpd.parser.Lexer.TIME:
-                timesig = Rat(toc.m_num, toc.m_den)
-                staff.append(Bar(timesig))
+                timesig = elems.TimeSignature(toc.m_num, toc.m_den)
+                score.add_bar(timesig)
             elif toc_type == u'~':
-                staff[-1][-1].m_tie = True
+                tie_is_in_the_air = True
             elif toc_type == u'|':
-                if staff[-1].m_keysig != bar_content:
-                    print "FIXME: barlines not ok."
-        return staff
+                if not score.voice11.is_bar_full():
+                    logging.warning(u"bar check in question failed.")
+        return score
             
  
 
@@ -958,7 +859,7 @@ class PercBaseClass(MpdParsable):
         except mpd.MpdException, e:
             self.complete_to_musicdata_coords(lessonfile_ref, e)
             raise
-        track = score.get_midi_events_as_percussion()[0]
+        track = mpd.score_to_percussion_tracks(score)[0]
         track.set_volume(cfg.get_int('config/preferred_instrument_volume'))
         track.prepend_bpm(lessonfile_ref.get_tempo()[0],
                           lessonfile_ref.get_tempo()[1])
